@@ -39,13 +39,13 @@ ggplot_heatmap <- function(xx,
                            row_text_angle = 0,
                            column_text_angle = 45,
                            scale_fill_gradient_fun =
-                           scale_fill_gradientn(
-                             colors = viridis(
-                               n = 256, alpha = 1, begin = 0,
-                               end = 1, option = "viridis"
+                             scale_fill_gradientn(
+                               colors = viridis(
+                                 n = 256, alpha = 1, begin = 0,
+                                 end = 1, option = "viridis"
+                               ),
+                               na.value = "grey50", limits = NULL
                              ),
-                             na.value = "grey50", limits = NULL
-                           ),
                            grid_color = NA,
                            grid_size = 0.1,
                            key.title = NULL,
@@ -57,11 +57,11 @@ ggplot_heatmap <- function(xx,
                            type = c("heatmap", "scatter"),
                            pointsize = 5,
                            point_size_mat = NULL,
-                           label_format_fun = function(...) format(..., digits = 4),
                            point_size_name = "Point size",
+                           label_format_fun = function(...) format(..., digits = 4),
                            custom_hovertext = NULL,
+                           showticklabels = c(TRUE, TRUE),
                            ...) {
-
   theme_clear_grid_heatmap <- theme(
     axis.line = element_line(color = "black"),
     panel.grid.major = element_blank(),
@@ -120,9 +120,11 @@ ggplot_heatmap <- function(xx,
         mdf[["text"]], "<br>",
         point_size_name, ": ", label_format_fun(mdf[[4]])
       )
-      aes_args <- list(color = paste_aes(val),
+      aes_args <- list(
+        color = paste_aes(val),
         text = "text",
-        size = paste_aes(point_size_name))
+        size = paste_aes(point_size_name)
+      )
 
       # geom_args[["mapping"]] <- aes_string(
       #   color = paste_aes(val),
@@ -131,8 +133,10 @@ ggplot_heatmap <- function(xx,
       # )
     } else {
       geom_args[["size"]] <- grid_size
-      aes_args <- list(color = paste_aes(val),
-        text = "text")
+      aes_args <- list(
+        color = paste_aes(val),
+        text = "text"
+      )
       # geom_args[["mapping"]] <- aes_string(color = paste_aes(val), text = "text")
     }
   }
@@ -151,17 +155,19 @@ ggplot_heatmap <- function(xx,
     theme_bw() +
     theme_clear_grid_heatmap +
     theme(
-      axis.text.x = element_text(
-        angle = column_text_angle,
-        size = fontsize_col,
-        hjust = 1
-      ),
-      axis.text.y = element_text(
-        angle = row_text_angle,
-        size = fontsize_row,
-        hjust = 1
-      ),
-      axis.title = element_blank()
+      axis.title = element_blank(),
+      axis.text.x = if (showticklabels[[1]]) {
+        element_text(angle = column_text_angle, size = fontsize_col, hjust = 1)
+      } else {
+        element_blank()
+      },
+      axis.ticks.x = if (showticklabels[[1]]) element_line() else element_blank(),
+      axis.text.y = if (showticklabels[[2]]) {
+        element_text(angle = row_text_angle, size = fontsize_row, hjust = 1)
+      } else {
+        element_blank()
+      },
+      axis.ticks.y = if (showticklabels[[2]]) element_line() else element_blank(),
     )
 
   if (type == "scatter") {
@@ -243,8 +249,10 @@ plotly_heatmap <- function(x,
                            colorbar_len = 0.3,
                            colorbar_thickness = 30,
                            custom_hovertext = NULL,
-                           showticklabels = c(TRUE, TRUE)) {
-
+                           point_size_mat = NULL,
+                           point_size_name = "Point size",
+                           showticklabels = c(TRUE, TRUE),
+                           label_format_fun = function(...) format(..., digits = 4)) {
   if (is.function(colors)) colors <- colors(256)
 
   if (is.null(label_names)) {
@@ -256,6 +264,9 @@ plotly_heatmap <- function(x,
   } else {
     assert_that(length(label_names) == 3)
   }
+  if (!is.null(point_size_mat)) {
+    point_size_mat <- as.matrix(point_size_mat)
+  }
 
   if (!is.null(custom_hovertext)) {
     text_mat <- custom_hovertext
@@ -264,24 +275,57 @@ plotly_heatmap <- function(x,
     text_mat[] <- lapply(
       seq_along(text_mat),
       function(i) {
-        paste0(
-          label_names[3], ": ", x[, i], "<br>",
+        lab <- paste0(
+          label_names[1], ": ", rownames(x), "<br>",
           label_names[2], ": ", colnames(x)[i], "<br>",
-          label_names[1], ": ", rownames(x)
+          label_names[3], ": ", label_format_fun(x[, i])
         )
+        if (!is.null(point_size_mat)) {
+          lab <- paste0(
+            lab, "<br>",
+            point_size_name, ": ", label_format_fun(point_size_mat[, i])
+          )
+        }
+        lab
       }
     )
     text_mat <- as.matrix(text_mat)
   }
 
-  p <- plot_ly(
-    z = x, x = 1:ncol(x), y = 1:nrow(x), text = text_mat,
-    type = "heatmap",
-    showlegend = FALSE,
-    colors = colors,
-    hoverinfo = "text",
-    zmin = limits[1], zmax = limits[2]
-  ) %>%
+  if (is.null(point_size_mat)) {
+    p <- plot_ly(
+      z = x, x = 1:ncol(x), y = 1:nrow(x), text = text_mat,
+      type = "heatmap",
+      showlegend = FALSE,
+      colors = colors,
+      hoverinfo = "text",
+      zmin = limits[1], zmax = limits[2]
+    )
+  } else {
+    melt <- function(x, cn = colnames(x), rn = rownames(x)) {
+      xdf <- reshape2::melt(x)
+      xdf$Var1 <- factor(xdf$Var1, levels = rn)
+      xdf$Var2 <- factor(xdf$Var2, levels = cn)
+      xdf
+    }
+    xdf <- melt(x)
+    tdf <- melt(text_mat)
+    pdf <- melt(point_size_mat)
+
+    p <- plot_ly(
+      x = as.numeric(xdf$Var1),
+      y = as.numeric(xdf$Var2),
+      text = tdf$value,
+      color = xdf$value,
+      size = pdf$value,
+      type = "scatter",
+      mode = "markers",
+      showlegend = FALSE,
+      colors = colors,
+      hoverinfo = "text"
+    )
+  }
+  p <- p %>%
     layout(
       xaxis = list(
         tickfont = list(size = fontsize_col),
@@ -319,7 +363,6 @@ plotly_dend <- function(dend,
                         side = c("row", "col"),
                         flip = FALSE,
                         dend_hoverinfo = TRUE) {
-
   if (is.hclust(dend)) {
     dend <- as.dendrogram(dend)
   }
@@ -378,8 +421,7 @@ plotly_dend <- function(dend,
           yaxis = axis2
         )
     }
-  }
-  else {
+  } else {
     add_plot_lines <- function(p) {
       p %>%
         add_segments(
@@ -429,7 +471,6 @@ ggplot_side_color_plot <- function(df,
                                    is_colors = FALSE,
                                    fontsize = 10,
                                    label_name = NULL) {
-
   type <- match.arg(type)
   if (is.matrix(df)) df <- as.data.frame(df)
   assert_that(is.data.frame(df))
@@ -483,6 +524,10 @@ ggplot_side_color_plot <- function(df,
   }
   theme <- list(common_theme, specific_theme)
 
+  if (is_colors) {
+    palette <- setNames(unique(df[["value"]]), unique(df[["value"]]))
+  }
+
   if (is.function(palette)) {
     palette <- setNames(palette(nlevels(df[["value"]])), levels(df[["value"]]))
   } else if (!all(levels(factor(df[["value"]])) %in% names(palette))) {
@@ -499,7 +544,7 @@ ggplot_side_color_plot <- function(df,
     scale_fill_manual(
       name = scale_title,
       breaks = levels(df[["value"]]),
-      values = palette[levels(df[["value"]])]
+      values = as.character(palette[levels(df[["value"]])])
     ) +
     theme
   return(g)
@@ -532,14 +577,25 @@ default_side_colors <- function(n) {
 ## Predict luminosity of cells and change text based on that
 ## http://stackoverflow.com/questions/12043187/how-to-check-if-hex-color-is-too-black
 predict_colors <- function(p,
-                           colorscale_df = p$x$data[[1]]$colorscale,
+                           colorscale_df = p$x$data[[1]]$colorscale %||% p$x$data[[2]]$marker$colorscale,
                            cell_values = p$x$data[[1]]$z,
                            plot_method = c("ggplot", "plotly")) {
-
   plot_method <- match.arg(plot_method)
 
+  if (is.null(cell_values)) {
+    cell_values <- as.data.frame(
+      matrix(
+        p$x$data[[1]]$marker$size,
+        ncol = length(unique(p$x$data[[1]]$x)),
+        nrow = length(unique(p$x$data[[1]]$y))
+      )
+    )
+  }
+  if (is.null(colorscale_df)) {
+    stop("Internal error in predicting colours!")
+  }
   cell_values <- as.data.frame(cell_values)
-  cell_values$row <- 1:nrow(cell_values)
+  cell_values$row <- seq_len(nrow(cell_values))
   cell_values_m <- reshape2::melt(cell_values, id.vars = "row")
   cell_values_vector <- cell_values_m$value
 
@@ -636,8 +692,8 @@ plotly_side_color_plot <- function(df,
                                    is_colors = FALSE,
                                    label_name = NULL,
                                    colorbar_len = 0.3,
-                                   fontsize = 10) {
-
+                                   fontsize = 10,
+                                   show_legend = TRUE) {
   type <- match.arg(type)
 
   if (is.null(label_name)) label_name <- type
@@ -648,7 +704,6 @@ plotly_side_color_plot <- function(df,
   #   data <- t(data)
   # }
   data <- as.data.frame(data, stringsAsFactors = TRUE)
-  data[] <- lapply(data, factor)
   data_vals <- unlist(data)
   levels <- levels(data_vals)
   levels <- levels[!is.na(levels)]
@@ -671,7 +726,10 @@ plotly_side_color_plot <- function(df,
   levs2nums <- setNames(seq_along(levels), levels)
 
   df_nums <- data
-  df_nums[] <- lapply(data, function(col) as.numeric(levs2nums[as.character(col)]))
+  df_nums[] <- lapply(
+    data,
+    function(col) as.numeric(levs2nums[as.character(col)])
+  )
   df_nums <- as.matrix(df_nums)
   if (type == "column") {
     df_nums <- t(df_nums)
@@ -679,7 +737,9 @@ plotly_side_color_plot <- function(df,
   if (ncol(df) == 1) {
     key_title <- colnames(df)
   } else {
-    key_title <- paste(gsub("^(\\w)", "\\U\\1", type, perl = TRUE), "annotation")
+    key_title <- paste(
+      gsub("^(\\w)", "\\U\\1", type, perl = TRUE), "annotation"
+    )
   }
 
   text_mat <- data
@@ -688,15 +748,15 @@ plotly_side_color_plot <- function(df,
     function(i) {
       if (type == "row") {
         paste0(
-          "value: ", data[, i], "<br>",
           "variable: ", colnames(data)[i], "<br>",
+          "value: ", data[, i], "<br>",
           label_name, ": ", rownames(data)
         )
       } else {
         paste0(
+          "variable: ", rownames(data),
           "value: ", data[, i], "<br>",
-          label_name, ": ", colnames(data)[i], "<br>",
-          "variable: ", rownames(data)
+          label_name, ": ", colnames(data)[i], "<br>"
         )
       }
     }
@@ -827,7 +887,7 @@ export <- function(x, file, width, height) {
   )
 }
 
-size_default <- function(file_extension, direction=c("width", "height")) {
+size_default <- function(file_extension, direction = c("width", "height")) {
   direction <- match.arg(direction)
   ## webshot uses viewport size in pixels to control file size, so
   ## all sizes in pixels
@@ -849,9 +909,10 @@ hmly_to_file <- Vectorize(hmly_to_file_1file, vectorize.args = "file")
 ## Copied from gplots
 #' @importFrom grDevices col2rgb
 col2hex <- function(col) {
-    colMat <- col2rgb(col)
-    rgb(red = colMat[1, ] / 255,
-        green = colMat[2, ] / 255,
-        blue = colMat[3, ] / 255)
+  colMat <- col2rgb(col)
+  rgb(
+    red = colMat[1, ] / 255,
+    green = colMat[2, ] / 255,
+    blue = colMat[3, ] / 255
+  )
 }
-
